package cds.aida.gui;
/* $Id: Conf.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Hashtable;
import java.util.ResourceBundle;

import javax.swing.JComponent;

/**
 *  Gives access to configuration properties of Conf.PROPERTIES_FILE
 *  @see cds.aida.gui.conf properties file
 *  @see L10N for localisation properties
 */
public class Conf extends PropertiesHandler {
	private final String PROPERTIES_FILE="cds.aida.gui.conf"; //$NON-NLS-1$
	private static Conf conf=null;
	// TODO: find a non-global way to do this:
	private JComponent graphicRoot=null;
	
	private Hashtable varconf;
	public static final String SERVER="server";
	public static final String SERVER_ENTRY="cgi.dir";
	public static final String WFSERVER="workflow.server.addr";
	
	public static final int CGI_USETOOL=1;
	public static final int CGI_GETLIST=2;
	
	/**
	 * @see getInstance
	 */
	private Conf() {
		try {
			rb=ResourceBundle.getBundle(PROPERTIES_FILE);
			varconf=new Hashtable();
		}
		catch(Exception e) {
			Logger logger=Logger.getInstance();
			logger.log(
					Logger.WARNING,
					"Cannot get resource bundle for "+ //$NON-NLS-1$
					PROPERTIES_FILE);
		}
		try {
			URLConnection.setContentHandlerFactory(
					new AidaContentHandlerFactory());
		}
		catch(Exception e) {
			Logger logger=Logger.getInstance();
			logger.log(
					Logger.ERROR,
					"Cannot init url content handler"); //$NON-NLS-1$
		}
	}
	
	/**
	 * Get instance of Conf singleton
	 * @return the only Conf instance
	 */
	public static synchronized Conf getInstance() {
		if(conf==null) {
			conf=new Conf();
		}
		return conf;
	}
	
	/**
	 * @return Returns the graphicRoot component of the application.
	 */
	public JComponent getGraphicRoot() {
		return graphicRoot;
	}
	/**
	 * Remember globally the graphic root component of the application,
	 * so that stuff like updating the UI components tree is possible,
	 * like LookAndFeel stuff.
	 * 
	 * @param graphicRoot The graphicRoot to set.
	 */
	public void setGraphicRoot(JComponent graphicRoot) {
		this.graphicRoot = graphicRoot;
	}

	/**
	 * Get UserAgent label used for http connections
	 * conf keys: useragent and version
	 * @return name
	 */
	public String getUserAgent() {
		String ua=get("useragent","Aida"); //$NON-NLS-1$ //$NON-NLS-2$
		String ver=get("version","0.1"); //$NON-NLS-1$ //$NON-NLS-2$
		return ua+"/"+ver; //$NON-NLS-1$
	}

	private String trimSlashes(String u) {
		String v=u.trim();
		v=v.replaceAll("^/*",""); //$NON-NLS-1$ //$NON-NLS-2$
		v=v.replaceAll("/*$",""); //$NON-NLS-1$ //$NON-NLS-2$
		return v;
	}
	
	public String get(String key) {
		return get(key,null);
	}
	public String get(String key, String default_value) {
		// check in varconf, then in properties
		Object v=varconf.get(key);
		return (v==null)?super.get(key,default_value):v.toString();
	}
	
	public void set(String key,String value) {
		// cannot put in properties, therefore put in varconf
		varconf.put(key,value);
	}
	
	/**
	 * get the Url to call aida services,
	 * based on configuration properties
	 * (server, cgi.usetool, etc.)
	 * @param service (USETOOL, GETLIST...)
	 * @return cgi url (null if malformed url from config parameters)
	 */
	public URL getAidaCgi(int service) {
		Logger logger=Logger.getInstance();
		L10N lion=L10N.getInstance();
		String url="http://"; //$NON-NLS-1$
		String sep="/"; //$NON-NLS-1$
		
		// Server name
		String srv=get(SERVER,""); //$NON-NLS-1$
		if("".equals(srv)) { //$NON-NLS-1$
			srv="127.0.0.1"; //$NON-NLS-1$
			String msg=lion.get(
					"config.usingdefaultserver", //$NON-NLS-1$
					"No server found in config, "+ //$NON-NLS-1$
					"using default server (localhost)"); //$NON-NLS-1$
			logger.log(Logger.WARNING,"Conf.getAidaCgi: "+msg); //$NON-NLS-1$
		}
		url+=trimSlashes(srv);
		
		// Cgi dir
		String cgidir=get(SERVER_ENTRY,"~aida/cgi-bin"); //$NON-NLS-1$ //$NON-NLS-2$
		url+=sep+trimSlashes(cgidir);
		
		// Service
		String serv=""; //$NON-NLS-1$
		switch(service) {
			case CGI_USETOOL:
				serv=get("cgi.usetool","useTool.cgi"); //$NON-NLS-1$ //$NON-NLS-2$
				break;
			case CGI_GETLIST:
				serv=get("cgi.getlist","getList.cgi"); //$NON-NLS-1$ //$NON-NLS-2$
				break;
			default:
				logger.log(
						Logger.ERROR,
						"Conf.getAidaCgi: unknown service("+service+")"); //$NON-NLS-1$ //$NON-NLS-2$
				return null;
		}
		url+=sep+trimSlashes(serv);

		URL u=null;
		try { u=new URL(url); } catch(MalformedURLException mue) {}
		return u;
	}
}
