package cds.aida.gui;
/* $Id: Logger.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.io.PrintStream;
import java.util.Vector;

/**
 * Logging facilities
 */
public class Logger {
	private static Logger logger=null;
	private PrintStream out,err;
	/** verbosity */
	private int v;
	private int DEFAULT_VERBOSE_LEVEL;
	public static final int ERROR=1;
	public static final int WARNING=3;
	public static final int INFO=5;
	public static final int DEBUG=7;
	public static final int TRACE=9;
	/** stock msg in vector */
	private boolean doStock;
	private Vector stock;
	private int stockVerbosity;
		
	/**
	 * @see getInstance
	 */
	private Logger() {
		out=System.out;
		err=System.err;
		String cnf;
		int lvl;
		stock=new Vector();
		
		Conf conf=Conf.getInstance();
		try {
			lvl=conf.getInt("logger.default_verbose_level",INFO); //$NON-NLS-1$
			DEFAULT_VERBOSE_LEVEL=lvl;
			log(Logger.DEBUG,"Logger: default_verbose_level("+lvl+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		catch(Exception e) {
			DEFAULT_VERBOSE_LEVEL=INFO;
		}

		try {
			lvl=conf.getInt("logger.verbose_level"); //$NON-NLS-1$
			setVerbosity(lvl);
		}
		catch(Exception e) {
			setVerbosity(ERROR);
		}
		
		try {
			doStock=Conf.getInstance().getBool("logger.stock",true); //$NON-NLS-1$
			stockVerbosity=conf.getInt("logger.stock_verbosity",DEBUG); //$NON-NLS-1$
		}
		catch(Exception e) {
			doStock=true;
			stockVerbosity=DEBUG;
		}
	}

	public static synchronized Logger getInstance() {
		if(logger==null) {
			logger=new Logger();
		}
		return logger;
	}
	
	/**
	 * Log the msg with the default verbosity level
	 * @param msg
	 */
	public void log(String msg) {
		this.log(DEFAULT_VERBOSE_LEVEL,msg);
	}
	
	/**
	 * Log the msg with the specified verbosity level
	 * @param v Verbosity level (Logger.ERROR, Logger.DEBUG...)
	 * @param msg
	 */
	public void log(int v, String msg) {
		if(getVerbosity()>=v) {
			outputLog(v,msg);
		}
		if(doStock && getStockVerbosity()>=v) {
			stockLog(v,msg);
		}
	}
	private void outputLog(int v, String msg) {
		// XXX add timestamps
		if(v==ERROR) { err.println("["+v+"] "+msg); }  //$NON-NLS-1$ //$NON-NLS-2$
		else { out.println("["+v+"] "+msg); }		 //$NON-NLS-1$ //$NON-NLS-2$
	}
	private void stockLog(int v, String msg) {
		stock.add("["+v+"] "+msg); //$NON-NLS-1$ //$NON-NLS-2$
	}
	public Vector getStock() {
		return stock;
	}
	
	/**
	 * @return Returns the verbosity level
	 */
	public int getVerbosity() {
		return v;
	}
	public int getStockVerbosity() {
		return stockVerbosity;
	}
	
	/**
	 * @param v The verbosity level to set.
	 */
	public void setVerbosity(int v) {
		log(
				Logger.DEBUG,
				"Logger.setVerbosity: level("+v+")"); //$NON-NLS-1$ //$NON-NLS-2$
		this.v=v;
	}
	/**
	 * change error and output streams
	 * @param out
	 * @param err
	 */
	public void setStreams(PrintStream out, PrintStream err) {
		this.err=err;
		this.out=out;
	}
}
	
