package cds.aida.gui;
/* $Id: Main.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.UIManager;

/**
 * Aida GUI main class, handling Look&Feel and command line parameters.
 * @see GlenView
 */
public class Main {

	static private void installLookAndFeel(String lnfName, String lnfClass) {
		ClassLoader cl=ClassLoader.getSystemClassLoader();
		try {
			cl.loadClass(lnfClass);
			UIManager.installLookAndFeel(lnfName, lnfClass);
		}
		catch(Throwable t) {
			Logger logger=Logger.getInstance();
			logger.log(
					Logger.DEBUG,
					"Main.installLookAndFeel: error installing"+ //$NON-NLS-1$
					"("+lnfClass+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	static private void installLookAndFeels() {
		L10N lion=L10N.getInstance();
		boolean setLnf=lion.getInt("lnf.set",new Integer(0))>0; //$NON-NLS-1$
		if(setLnf) {
			// TODO: autodetect available L&F, how ?
			// TODO: remove windows L&F on non windows platform
			// (but this one is bugfixed in later jdk iirc)
			ClassLoader cl=ClassLoader.getSystemClassLoader();
			installLookAndFeel(
				"GTK", //$NON-NLS-1$
				"com.sun.java.swing.plaf.gtk.GTKLookAndFeel"); //$NON-NLS-1$
			installLookAndFeel(
				"Liquid", //$NON-NLS-1$
				"com.birosoft.liquid.LiquidLookAndFeel"); //$NON-NLS-1$
			installLookAndFeel(
				"Tonic", //$NON-NLS-1$
				"com.digitprop.tonic.TonicLookAndFeel"); //$NON-NLS-1$
			installLookAndFeel(
				"Plastic", //$NON-NLS-1$
				"com.jgoodies.looks.plastic.PlasticLookAndFeel"); //$NON-NLS-1$
//			installLookAndFeel(
//			"Napkin",
//			"napkin.NapkinLookAndFeel");

			String lnf=""; //$NON-NLS-1$
			try {
				lnf=lion.get(
						"lnf.use" //$NON-NLS-1$
						,"com.birosoft.liquid.LiquidLookAndFeel"); //$NON-NLS-1$
				UIManager.setLookAndFeel(
						lnf);
			}
			catch(Throwable t) {
				Logger.getInstance().log(
						Logger.DEBUG
						,"Main.installLookAndFeels: error setting default("+lnf+")"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		else {
			try {
				// unnecessary
				UIManager.setLookAndFeel(
						UIManager.getCrossPlatformLookAndFeelClassName());
			}
			catch(Throwable t) {
				Logger.getInstance().log(
						Logger.TRACE
						,"Main.installLookAndFeels: error setting cpf LnF"); //$NON-NLS-1$
			}
		}
	}
	
	public static void main(String[] args) {
		installLookAndFeels();
		L10N lion=L10N.getInstance();
		Conf conf=Conf.getInstance();
		Logger logger=Logger.getInstance();

		// args
		String argi;
		int xgeom=0,ygeom=0;
		boolean setgeometry=false;
		for(int i=0;i<args.length;i++) {
			argi=args[i].toLowerCase();
			
			if(argi.startsWith("-server=")) {
				String server=argi.substring(argi.indexOf('=')+1);
				conf.set(Conf.SERVER,server);
				logger.log(
						Logger.INFO,
						"Main: args: server("+ //$NON-NLS-1$ 
						server+")"); //$NON-NLS-1$
				continue;
			}
			if(argi.startsWith("-serverentry=")) {
				String entry=argi.substring(argi.indexOf('=')+1);
				conf.set(Conf.SERVER_ENTRY,entry);
				logger.log(
						Logger.INFO,
						"Main: args: serverentry("+ //$NON-NLS-1$ 
						entry+")"); //$NON-NLS-1$
				continue;
			}
			if(argi.startsWith("-wfserver=")) {
				String server=argi.substring(argi.indexOf('=')+1);
				conf.set(Conf.WFSERVER,server);
				logger.log(
						Logger.INFO,
						"Main: args: wfserver("+ //$NON-NLS-1$ 
						server+")"); //$NON-NLS-1$
				continue;
			}
			
			if(argi.startsWith("-geometry=")) {
				String geo=argi.substring(argi.indexOf('=')+1);
				String[]xy=geo.split("x");
				if(xy.length!=2) {
					logger.log(
							Logger.WARNING,
							"Main: args: geometry malformed"); //$NON-NLS-1$
				}
				else {
					try {
						xgeom=Integer.parseInt(xy[0]);
						ygeom=Integer.parseInt(xy[1]);
						setgeometry=true;
						logger.log(
								Logger.INFO,
								"Main:args: geometry("+ //$NON-NLS-1$
								xgeom+","+ //$NON-NLS-1$
								ygeom+")"); //$NON-NLS-1$
					}
					catch(NumberFormatException nfe) {
						logger.log(
								Logger.WARNING,
								"Main: args: geometry malformed (nfe)"); //$NON-NLS-1$
					}
				}
				continue;
			}
		}
		
		// standard exitable main window
		JFrame f=new JFrame();
		f.getContentPane().setLayout(new BorderLayout());
		f.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		
		// that we baptise, then make grown up
		f.setTitle(lion.get("main.name","Aida")); //$NON-NLS-1$ //$NON-NLS-2$
		Dimension d;
		if(setgeometry) {
			d=new Dimension(xgeom,ygeom);
		}
		else {
			d=lion.getDimension(
					"main.size", //$NON-NLS-1$
					new Dimension(742,542));
		}
		f.setSize(d);
		
		// the whole interface
		GlenView gv=new GlenView();
		f.getContentPane().add(gv, BorderLayout.CENTER);
		// from whom we remember the L&F top component:
		conf.setGraphicRoot(f.getRootPane());
		// and here we go:
		f.show();
	}
	
}
