package cds.aida.gui;
/* $Id: TListView.java,v 1.2 2007/06/06 12:24:19 beugnot Exp $ */
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.EventListenerList;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import cds.aida.tool.Tool;
import cds.aida.tool.ToolsList;

/**
 * View for the model Tools, which list the available tools
 * <p>provides tree view and buttons to clear/refresh
 */
public class TListView extends JPanel
implements  ActionListener, TreeSelectionListener, MouseListener,Observer {
	private ToolsList tools;
	private EventListenerList listeners;
	// visu.
	private JButton bGetList, bResetList;
	private JToolsTree tree;
	
	/**
	 * @param tools
	 */
	public TListView(ToolsList tools) {
		super();
		this.tools = tools;
		tools.addObserver(this);
		listeners=new EventListenerList();
		
		// tree view and listeners
		tree=new JToolsTree();
		tree.setShowsRootHandles(true);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addTreeSelectionListener(this);
		TreeModel tm=tools.getTree();
		tree.setModel(tm);
		tree.addMouseListener(this);
		JScrollPane treepane=new JScrollPane(tree);
				
		// button bar and listeners
		L10N lion=L10N.getInstance();
		bGetList=new JButton(lion.get("tools.getlist","Refresh")); //$NON-NLS-1$ //$NON-NLS-2$
		bGetList.addActionListener(this);
		bResetList=new JButton(lion.get("tools.resetlist","Reset")); //$NON-NLS-1$ //$NON-NLS-2$
		bResetList.addActionListener(this);
		JPanel actions=new JPanel(new GridLayout(1,2));
		actions.add(bGetList);
		actions.add(bResetList);
		// composition
		setLayout(new BorderLayout());
		add(actions,BorderLayout.NORTH);
		add(treepane,BorderLayout.CENTER);
	}


	
	/**
	 * Observer implementation
	 */
	public void update(Observable o, Object arg) {
		if(arg.equals(tools)) {
			Logger.getInstance().log(Logger.TRACE,
					"ToolsView.update by Tools"); //$NON-NLS-1$
			tree.setModel(tools.getTree());
			tree.validate();
			tree.repaint();
			// XXX: L&F hasn't changed, but validate/repaint
			// don't get it right otherwise:
			tree.updateUI();
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		Object o=e.getSource();
		if(o.equals(bGetList)) {
			if(!tools.fetchList()) {
				L10N lion=L10N.getInstance();
				String refresh=lion.get("tools.getlist","Refresh"); //$NON-NLS-1$ //$NON-NLS-2$
				String rferror=lion.get(
						"tools.getlist.errormsg", //$NON-NLS-1$
						"Error when fetching tools list\nCheck configuration file"+ //$NON-NLS-1$
						"and server"); //$NON-NLS-1$
				JOptionPane.showMessageDialog(
						this,
						rferror,
						refresh,
						JOptionPane.ERROR_MESSAGE
						);
			}
		}
		else if(o.equals(bResetList)) {
			tools.resetList();
			tree.repaint();
		}
	}
	
	public void addTreeListener(TreeSelectionListener tsl){
		tree.addTreeSelectionListener(tsl);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		//TreePath path=tree.getPathForLocation(e.getX(), e.getY());
		TreePath path=tree.getLeadSelectionPath();
		if(path!=null) {
			if(e.getClickCount()==1) {
			}
			else if(e.getClickCount()==2) {
				Object o=path.getLastPathComponent();
				if(o==null) { return; }
				Object n=((DefaultMutableTreeNode)o).getUserObject();
				if(n instanceof Tool) {
					Logger.getInstance().log(
							Logger.TRACE,
							"ToolsView.fireToolSelected("+n+")"); //$NON-NLS-1$ //$NON-NLS-2$
					//fireToolSelected(new ToolsEvent(this,(Tool)n));
				}
				else {
					Logger.getInstance().log(
							Logger.TRACE,
							"ToolsView.mousePressed: Not a Tool:"+n);							 //$NON-NLS-1$
				}
			}
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
	}
}
