package cds.aida.gui;
/* $Id: WS1View.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.awt.BorderLayout;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.Border;

import cds.aida.tool.Tool;
import cds.aida.w1.ToolView;
import cds.aida.w1.ToolViewFactory;
import cds.aida.w1.WS1;

/**
 *  View for a WS1, which if a workspace containing only one tool.
 *  <p>GUI for the input/output/execution of the tool selected in the WS1
 */
public class WS1View extends JPanel
implements WSView, DropTargetListener {
	/** workspace model for this view */
	protected WS1 ws;
	private Action
	actionLaunch, actionResetFields, actionShowCgiRequest;
	// visu
	JButton launch,resetf,showcgiurl;
	private Border compborder=BorderFactory.createEmptyBorder(10,10,10,10);
		
	public WS1View(WS1 ws) {
		// set the workspace which is viewed and observed
		this.ws=ws;
		ws.addGraphDescriptorListener(new CustomGDListener());
		this.initActions();
		
		// content until a tool is added
		JLabel pname=new JLabel();
		pname.setText(L10N.getInstance().get("ws1.name", "Workspace WS1")); //$NON-NLS-1$ //$NON-NLS-2$
		pname.setBorder(compborder);
		setLayout(new BorderLayout());
		add(pname, BorderLayout.NORTH);
		
		// drag'n'drop facilities
		new DropTarget(
				this/* is the target component*/,
		        DnDConstants.ACTION_COPY_OR_MOVE,
		        this/* drop target is listening*/
				);
	}

	private void initActions() {
		ActionMap am=getActionMap();
		InputMap im=getInputMap();
		actionLaunch=new ActionLaunch();
		am.put("launch",actionLaunch); //$NON-NLS-1$
		actionResetFields=new ActionResetFields();
		am.put("resetfields",actionResetFields); //$NON-NLS-1$
		actionShowCgiRequest=new ActionShowCgiRequest();
		am.put("showcgirequest",actionShowCgiRequest); //$NON-NLS-1$
	}
	class ActionLaunch extends AbstractAction {
		public ActionLaunch() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("tool.run.label","Launch")); //$NON-NLS-1$ //$NON-NLS-2$
			//putValue(MNEMONIC_KEY,new Integer(KeyEvent.VK_L));
			putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke("F5")); //$NON-NLS-1$
		}
		public void actionPerformed(ActionEvent e) {
			// XXX should be in WS1 rather than WS1View ?
			// or even elsewhere ?
			Tool t=(Tool)ws.getTool();
			launch.setEnabled(false);
			resetf.setEnabled(false);
			new Thread(t).start();
		}
	}
	class ActionShowCgiRequest extends AbstractAction {
		public ActionShowCgiRequest() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("ws1.showcgirequest","Show Request")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		public void actionPerformed(ActionEvent e) {
			Tool t=(Tool)ws.getTool();
			String req=t.getCgiRequest();
			JFrame f=new JFrame();
			JPanel p=new JPanel();
			p.setLayout(new BorderLayout());
			JTextField tf=new JTextField(req);
			p.add(tf,BorderLayout.CENTER);
			f.getContentPane().add(p);
			f.pack();
			f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			f.setVisible(true);
		}
	}
	class ActionResetFields extends AbstractAction {
		public ActionResetFields() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("ws1.resetfields.label","Clear")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		public void actionPerformed(ActionEvent e) {
			ws.getTool().reset();
		}
	}
	

	/**
	 * GUI for actions (launch tool, etc.)
	 * @return Toolbar
	 */
	protected JToolBar actionsToolBar() {
		launch=new JButton(actionLaunch);
		resetf=new JButton(actionResetFields);
		showcgiurl=new JButton(actionShowCgiRequest);
		JToolBar actions=new JToolBar(JToolBar.HORIZONTAL);
		actions.setFloatable(false);
		actions.setOpaque(false);
		actions.add(launch);
		actions.add(resetf);
		actions.add(showcgiurl);
		return actions;
	}
	
	/**
	 * wrap the toolInterface in a scrollable pane
	 * @return GUI
	 */
	protected JScrollPane scrollableToolInterface() {
		ToolView tv=ToolViewFactory.create(ws.getTool());
		JScrollPane sp=new JScrollPane(tv);
		// sets mouse wheel scroll speed
		sp.getVerticalScrollBar().setUnitIncrement(10);
		// starts on top of pane
		sp.getVerticalScrollBar().setValue(0);
		return sp;
	}
	
	/**
	 * update when notified by the observable workspace
	 */
	public void update() {
		Logger.getInstance().log(
				Logger.TRACE,
				"WS1View.update("+getClass()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		
		
			// iokdbg: remove/recreate shouldnt be the updating process
			this.removeAll();
			JScrollPane sp=scrollableToolInterface();
			JToolBar tb=actionsToolBar();
			this.add(tb,BorderLayout.NORTH);
			this.add(sp,BorderLayout.CENTER);
			Conf.getInstance().getGraphicRoot().updateUI();
			// XXX right place to do this ? how to unregister ?
			Tool t=(Tool)ws.getTool();
		
		

	}
	
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dragEnter(DropTargetDragEvent dtde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
	 */
	public void dragExit(DropTargetEvent dte) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dragOver(DropTargetDragEvent dtde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
	 */
	public void drop(DropTargetDropEvent dtde) {
		// drag'n'drop facilities
		try {
			// only a Tool is dropable here
			DataFlavor df=new Tool.ToolDataFlavor();
			Transferable tr=dtde.getTransferable();
			if(dtde.isDataFlavorSupported(df)) {
				Tool tool=(Tool)tr.getTransferData(df);
				dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				// effective drop in the workspace
				ws.addTool(tool);
				update();
				dtde.dropComplete(true);
			}
			else {
				dtde.rejectDrop();
			}
		}
		catch(IOException ioe) {
			ioe.printStackTrace();
		}
		catch(UnsupportedFlavorException ufe) {
			ufe.printStackTrace();
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}
	/**
	 * @return Returns the ws.
	 */
	public WS1 getWs() {
		return ws;
	}
	/**
	 * @param ws The ws to set.
	 */
	public void setWs(WS1 ws) {
		this.ws = ws;
		//update(null,null);
	}
}
