package cds.aida.tool;
/* $Id: Tool.java,v 1.4 2007/06/06 12:24:19 beugnot Exp $ */

import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import cds.aida.gui.Conf;
import cds.aida.gui.L10N;
import cds.aida.gui.Logger;
import cds.aida.w1.ToolCallStatus;
import cds.aida.wsJlow.WSJ;
import cds.jlow.client.descriptor.Attributs;
import cds.jlow.descriptor.Constant;
import cds.jlow.descriptor.IPortDescriptor;
import cds.jlow.descriptor.TaskDescriptor;
import cds.jlow.util.Type;
import cds.jlow.util.Utils;

/**
 * Model for a tool.
 * <p>provides tool description, i/o lists, call method, call status...
 * <p>rq: also a Jlow TaskDescriptor
 * @see ToolIO, DefaultToolView
 */
public class Tool extends TaskDescriptor
implements Transferable, Runnable{
	private String categorie;
	private String comment;

	
	//used for the workspace one tool
	private ObservableTool obs;
	private ToolCallStatus callStatus=ToolCallStatus.CALL_NOTCALLED;
	private String digest;
	private ToolPort stdout;
	private ToolPort stderr;
	/** inputs  */private Vector i;
	/** outputs */private Vector o;
	
	
	DataFlavor[] flavors={
			DataFlavor.stringFlavor,
			new Tool.ToolDataFlavor()
	};
	
	public Tool(){
		super();
		obs=new ObservableTool(this);
		stdout=new ToolPort(id,L10N.getInstance().get("Tool.stdout.label"),L10N.getInstance().get("Tool.stdout.label"));
		stderr=new ToolPort(id,L10N.getInstance().get("Tool.stderr.label"),L10N.getInstance().get("Tool.stdout.label"));
		stdout.setData(new Constant());
		stderr.setData(new Constant());
		i=new Vector();
		o=new Vector();
		setIdOT(id);
		setLabel(getName());
	}
	
	public Tool(String id){
		super(id,Utils.getIdUnique());
		obs=new ObservableTool(this);
		stdout=new ToolPort(id,L10N.getInstance().get("Tool.stdout.label"),L10N.getInstance().get("Tool.stdout.label"));
		stderr=new ToolPort(id,L10N.getInstance().get("Tool.stderr.label"),L10N.getInstance().get("Tool.stdout.label"));
		stdout.setData(new Constant());
		stderr.setData(new Constant());
		i=new Vector();
		o=new Vector();
		setIdOT(id);
	}
	
	public Tool(String id,String name){
		super(id,name);
		obs=new ObservableTool(this);
		stdout=new ToolPort(id,L10N.getInstance().get("Tool.stdout.label"),L10N.getInstance().get("Tool.stdout.label"));
		stderr=new ToolPort(id,L10N.getInstance().get("Tool.stderr.label"),L10N.getInstance().get("Tool.stdout.label"));
		stdout.setData(new Constant());
		stderr.setData(new Constant());
		i=new Vector();
		o=new Vector();
		setIdOT(id);
	}
	
	public Tool(String id, String name, String comment){
		super(id,name);
		obs=new ObservableTool(this);
		setComment(comment);
		stdout=new ToolPort(id,L10N.getInstance().get("Tool.stdout.label"),L10N.getInstance().get("Tool.stdout.label"));
		stderr=new ToolPort(id,L10N.getInstance().get("Tool.stderr.label"),L10N.getInstance().get("Tool.stdout.label"));
		stdout.setData(new Constant());
		stderr.setData(new Constant());
		i=new Vector();
		o=new Vector();
		setIdOT(id);
	}
	
	/**
	 * const. and basic settings
	 * @param id
	 * @param label
	 * @param comment
	 */
	public Tool(String id, String name, Hashtable mods, String comment) {
		super(id,name,mods);
		obs=new ObservableTool(this);
		setComment(comment);
		stdout=new ToolPort(id,L10N.getInstance().get("Tool.stdout.label"),L10N.getInstance().get("Tool.stdout.label"));
		stderr=new ToolPort(id,L10N.getInstance().get("Tool.stderr.label"),L10N.getInstance().get("Tool.stdout.label"));
		stdout.setData(new Constant());
		stderr.setData(new Constant());
		i=new Vector();
		o=new Vector();
		setIdOT(id);
	}
	
	/**
	 * @return Returns the name of tool.
	 */
	public String toString(){
		return getName();
	}
		
	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}
	/**
	 * @param comment The comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}
	
	public String getLabel(){
		return getName();
	}
	
	public void setLabel(String label) {
		if(label==null||label.equals("")) { //$NON-NLS-1$
			label=L10N.getInstance().get("tool.noname"); //$NON-NLS-1$
			label+="("+this.getID()+")"; //$NON-NLS-1$ //$NON-NLS-2$
		}
		setName(label);
		// chg label in taskdescriptor ?
	}
	
	public Vector getListIPort(){
		return i;
	}
	

	
	public void setInputPort(int index,ToolPort tp){
		String id = Utils.getIdUnique();
		int miss=index-countInput();
		while(miss>0) {
			i.add(new ToolPort(id));
			miss--;
		}
		tp.setIndex(index-1);
		i.setElementAt(tp,index-1);
	}
	
	public Vector getListOPort(){
		return o;
	}
	
	public void setOutputPort(int index,ToolPort tp){
		String id = Utils.getIdUnique();
		int miss=index-countOutput();
		while(miss>0) {
			o.add(tp);
			miss--;
		}
		tp.setIndex(index-1);
		o.setElementAt(tp, index-1);
	}
	
	/**
	 * @param index
	 * @return output
	 */
	public ToolPort getOutput(int index) {
		if(o.size()<index) {
			return null;
		}
		return (ToolPort)o.get(index-1);
	}
	
	public ToolPort getInput(int index) {
		if(i.size()<index) {
			return null;
		}
		return (ToolPort)i.get(index-1);
	}
	
	/**
	 * Parse each lines of the descriptor,
	 * via parseDescLine
	 * @param desc
	 * @return success
	 */
	public boolean parseDesc(String desc) {
		boolean parsed_ok=true,parsed;
		String lines[]=desc.split("\n"); //$NON-NLS-1$
		for(int i=0;i<lines.length;i++) {
			parsed=parseDescLine(lines[i]);
			parsed_ok=parsed_ok && parsed;
		}
		Logger.getInstance().log(
				Logger.TRACE,
				"Tool.parseDesc("+ //$NON-NLS-1$
				lines.length+" lines - "+ //$NON-NLS-1$
				parsed_ok+")"); //$NON-NLS-1$
		return parsed_ok;
	}

	protected boolean parseDescLineTool(String key[],String value) {
		Logger logger=Logger.getInstance();
		if(key[1].equals("label")) { //$NON-NLS-1$
			this.setName(value);
			//nom tache
		}
		else if(key[1].equals("comment")) { //$NON-NLS-1$
			this.setComment(value);;
		}
		else if(key[1].equals("categories")) { //$NON-NLS-1$
			//return parseDescLineCategories(value);
		}
		else if(key[1].equals("flags")) { //$NON-NLS-1$
			// XXX: setFlag
		}
		else {
			logger.log(
					Logger.WARNING,
					// XXX missing whole line to log properly
					"Tool.parseDescLine: tool key("+key[1]+") not handled"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		return true;
	}
	protected boolean parseDescLineIO(String key[],String value,boolean isInput){
		Logger logger=Logger.getInstance();
		if(key.length<3) {
			// line like "(in|out)put.[0-9]+.[a-z.]+" -> at least 3 fields
			logger.log(
					Logger.WARNING,
					// XXX missing whole line to log properly
					"Tool.parseDescLine: key of input("+""+") incomplete"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return false;
		}
		
		// XXX validate index
		int idx=new Integer(key[1]).intValue();
		ToolPort t;
		if(isInput){
			t = this.getInput(idx);
		}else{
			t = this.getOutput(idx);
		}
		if(t==null) {
			t=new ToolPort(Utils.getIdUnique());
			if(isInput){
				t.setPosition(IPortDescriptor.INPUT);
			}else{
				t.setPosition(IPortDescriptor.OUTPUT);
			}
		}
		if(key[2].equals("type")) { //$NON-NLS-1$
			t = initPortType(t,value);
		}
		else if(key[2].equals("default")) { //$NON-NLS-1$
			t.setDefaultValue(value);
		}
		else if(key[2].equals("label")) { //$NON-NLS-1$
			t.setName(value);
		}
		else if(key[2].equals("comment")) { //$NON-NLS-1$
			t.setComment(value);
		}
		else {
			logger.log(
					Logger.WARNING,
					"Tool.parseDescLine: input("+key[2]+") subkey unhandled"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		logger.log(
				Logger.TRACE,
				"Tool.parseDescLine(ti:"+idx+"/"+this.countInput()+":"+ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				t.getName()+")"); //$NON-NLS-1$
		if(isInput) {
			this.setInputPort(idx,t);
		}
		else {
			this.setOutputPort(idx,t);
		}
		
		return true;
	}
	
	public ToolPort initPortType(ToolPort tp,String type) {	
		String t = type.toLowerCase();
	
		if(t.startsWith("fits")) {
			tp.setData(new Constant(Utils.getIdUnique(),new Type("fits"),new Vector()));
		}
		else if(t.startsWith("numeric")) {
			if(t.startsWith("numeric:int")){
				tp.setData(new Constant(Utils.getIdUnique(),new Type(t),new Integer(-0)));
			}else if(t.startsWith("numeric:float")){
				tp.setData(new Constant(Utils.getIdUnique(),new Type(t),new Float(-0)));
			}else{
				tp.setData(new Constant(Utils.getIdUnique(),new Type(t),new Double(-0)));
			}
		}
		else if(t.startsWith("data")) {
			tp.setData(new Constant(Utils.getIdUnique(),new Type(t),new Vector()));
		}
		else if(t.startsWith("image")) {
			tp.setData(new Constant(Utils.getIdUnique(),new Type(t),new Vector()));
		}
		else if(t.startsWith("bool")) {
			tp.setData(new Constant(Utils.getIdUnique(),Type.BOOLEAN,new Boolean(null)));
		}
		return tp;
	}
	
	/**
	 * Parse a key=value line from the descriptor of the tool,
	 * which impact the Tool model accordingly 
	 * 
	 * @param l line of the descriptor
	 * @return parsing success
	 */
	protected boolean parseDescLine(String l) {
		Logger logger=Logger.getInstance();
		boolean isInput=false; // used when parsing input/output similarly

		// key=value
		String kv[]=l.split("=",2); //$NON-NLS-1$
		if(kv.length!=2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDescLine: line("+l+") badly formed"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		String value=kv[1].trim();
		String key[]=kv[0].trim().split("[.]"); //$NON-NLS-1$
		if(key.length<2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDescLine: key of line("+l+") badly formed "+ //$NON-NLS-1$ //$NON-NLS-2$
					"(missing subkey?)"); //$NON-NLS-1$
			return false;
		}
		
		// tool.xxx
		if(key[0].equals("tool")) { //$NON-NLS-1$
			return parseDescLineTool(key,value);
		}
		// input.xxx or output.xxx
		if((isInput=key[0].equals("input"))||(key[0].equals("output"))) { //$NON-NLS-1$ //$NON-NLS-2$
			return parseDescLineIO(key,value,isInput);
		}
		// unknown
		logger.log(
				Logger.WARNING,
				"Tool.parseDescLine: line("+l+") not handled, skip"); //$NON-NLS-1$ //$NON-NLS-2$
		return false;
	}

	/**
	 * DataFlavor so that Tool is Transferable 
	 */
	public static class ToolDataFlavor extends DataFlavor {
		public ToolDataFlavor() {
			super(Tool.class, "Tool"); //$NON-NLS-1$
		}
	}
	
	public Object getTransferData(DataFlavor flavor) 
	throws UnsupportedFlavorException, IOException {
		if(flavor.equals(DataFlavor.stringFlavor)) {
			return this.getID();
		}
		else {
			return this;
		}
	}

	public DataFlavor[] getTransferDataFlavors() {
		// TODO Auto-generated method stub
		return flavors;
	}

	public boolean isDataFlavorSupported(DataFlavor flavor) {
		int j=flavors.length;
		for(int i=0;i<j;i++) {
			if(flavor.equals(flavors[i])) {
				return true;
			}
		}
		return false;
	}
	
	//This functions is used for the workspace one tool
	
	public ToolPort stdout() {
		return stdout;
	}
	public ToolPort stderr() {
		return stderr;
	}
	
	public int countInput() {
		return i.size();
	}
	
	public int countOutput() {
		return o.size();
	}
	
	private String encodeInputValue(String s) {
		String senc;
		try {
			senc=URLEncoder.encode(s,"utf-8"); //$NON-NLS-1$
		}
		catch(UnsupportedEncodingException uee) {
			// XXX try more ?
			senc=s;
		}
		return senc;
	}
	

	/**
	 * reset every IO, and stdout/err
	 */
	public void reset() {
		digest=""; //$NON-NLS-1$
		stdout.clearValues();
		stderr.clearValues();
		callStatus=ToolCallStatus.CALL_NOTCALLED;
		resetInputs();
		resetOutputs();
		obs.ping();
	}
	

	
	protected void resetInputs() {
		Vector v;
		Enumeration e;
		v=getListIPort();
		e=v.elements();
		while(e.hasMoreElements()) {
			ToolPort io=(ToolPort)e.nextElement();
			io.clearValues();
		}	
		// obs.ping() will be done at the ToolIO level
	}
	 

	
	protected void resetOutputs() {
		Vector v;
		Enumeration e;
		v=getListOPort();
		e=v.elements();
		while(e.hasMoreElements()) {
			ToolPort io=(ToolPort)e.nextElement();
			io.clearValues();
		}
		stdout.clearValues();
		stderr.clearValues();
		// obs.ping() will be done at the ToolIO level
	}
	
	/**
	 * @return cgi request string (server+params)
	 */
	public String getCgiRequest() {
		Logger logger=Logger.getInstance();
		Conf conf=Conf.getInstance();
		String id=getID();
		URL callerurl=conf.getAidaCgi(Conf.CGI_USETOOL); // XXX monosrv
		String caller=callerurl.toString();
		caller+="?tool.id="+id; //$NON-NLS-1$
		
		Iterator iter=i.iterator();
		ToolPort ti;
		int tin=1;
		while(iter.hasNext()) {
			ti=(ToolPort)iter.next();
			if(ti.isReady()) {
				// Construct call string of input.X
				// possibly multi-valued
				if(ti.isMultivalued()) {
					Enumeration tivals=ti.getValues();
					int tivi=1;
					while(tivals.hasMoreElements()) {
						String tival=tivals.nextElement().toString();
						String tivalenc=encodeInputValue(tival);
						caller+="&input."+tin+"."+tivi+"="+tivalenc; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						tivi++;
					}
				}
				else {
					String tival=ti.getFirstValue().toString();
					String tivalenc=encodeInputValue(tival);
					caller+="&input."+tin+"="+tivalenc; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			else {
				logger.log(Logger.TRACE,"Tool.call: input("+tin+") not set"); //$NON-NLS-1$ //$NON-NLS-2$
				setCallStatus(ToolCallStatus.CALL_INPUT_NOT_READY);
				return ""; //$NON-NLS-1$
			}
			tin++;
		}
		return caller;
	}
	
	/**
	 * Parse a digest (text returned after a call),
	 * via parseDigestLine
	 * @return parsing success
	 */
	protected boolean parseDigest() {
		Logger logger=Logger.getInstance();
		// iokdbg: resetOutputs here because of possiblity of multiple launch
		// should probably be handled before this point, thought. 
		resetOutputs();
		if(digest==null||digest.equals("")) { //$NON-NLS-1$
			logger.log(
					Logger.ERROR,
					"Tool.parseDigest:empty"); //$NON-NLS-1$
			return false;
		}
		logger.log(Logger.DEBUG, "Tool.parseDigest(\n"+digest+")"); //$NON-NLS-1$ //$NON-NLS-2$
		try {
			boolean parsed_ok=true;
			BufferedReader br=new BufferedReader(new StringReader(digest));
			String l=br.readLine();

			while(l!=null) {
				l=l.trim();
				if(l.equals("")) { //$NON-NLS-1$
					l=br.readLine();
					continue;
				}
				// --- mark the beginning of debug/verbose infos, if any.
				if(l.startsWith("---")) { //$NON-NLS-1$
					break;
				}
				parsed_ok=parsed_ok && parseDigestLine(l);
				l=br.readLine();
			}
			return parsed_ok;
		}
		catch(IOException ioe) {
			logger.log(
					Logger.ERROR,
					"Tool.parseDigest:IOException"); //$NON-NLS-1$
			setCallStatus(ToolCallStatus.CALL_PARSERESULTS_ERROR);
			// XXX handle more precisely ?
			return false;
		}
	}
	
	/**
	 * parse a digest line, sets the tool output accordingly
	 * @param l - digest line
	 * @return true on parsing success
	 */
	protected boolean parseDigestLine(String l) {
		// multiple return point on error
		boolean parsed_ok=true;
		Logger logger=Logger.getInstance();
		
		if(l.startsWith("Error")) { // XXX hardcoded //$NON-NLS-1$
			logger.log(
					Logger.ERROR,
					"Tool.parseDigestLine: "+l); //$NON-NLS-1$
			// XXX error transmission
			return true;
		}
		// split key=value
		String kv[]=l.trim().split("=",2); //$NON-NLS-1$
		if(kv.length!=2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: line("+l+") badly formed"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		String value=kv[1];
		
		// split key.subkey
		String key[]=kv[0].split("[.]"); //$NON-NLS-1$
		if(key.length<2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: key of line("+l+") badly formed"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}

		if(key[0].equals("output")) { //$NON-NLS-1$
			if(value.startsWith("FILE_SEQUENCE")) { //$NON-NLS-1$
				// skip this one, values digest lines will be parsed latter
				return parsed_ok;
			}
			
			if(key[1].equals("status")) { //$NON-NLS-1$
				// XXX output.status handling
				logger.log(
						Logger.INFO,
						"Tool.parseDigestLine: Status("+value+")"); //$NON-NLS-1$ //$NON-NLS-2$
				return true;
			}
			if(key[1].equals("stdout")) { //$NON-NLS-1$
				stdout.addValue(value);
			}
			else if(key[1].equals("stderr")) { //$NON-NLS-1$
				stderr.addValue(value);
			}
			else {
				// TODO: chk if subkeys
				int idx;
				try {
					idx=new Integer(key[1]).intValue();
					this.getOutput(idx).addValue(value);
				}
				catch(Exception e) {
					// NumberFormatException if index malformed
					// NullPointerException if output(idx) out of bounds
					logger.log(
							Logger.ERROR,
							"Tool.parseDigestLine: index("+key[1]+") error"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		else {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: line("+l+") unknown, skip."); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return parsed_ok;
	}
	
	public ToolCallStatus getCallStatus() {
		return callStatus;
	}
	protected void setCallStatus(ToolCallStatus c) {
		Logger.getInstance().log(
				Logger.TRACE,
				"Tool.setCallStatus("+c+")"); //$NON-NLS-1$ //$NON-NLS-2$
		callStatus=c;
		obs.ping();
	}
	/**
	 * @return cgi request string (server+params)
	 */

	
	public void run() {
		Logger logger=Logger.getInstance();
		Conf conf=Conf.getInstance();
		resetOutputs();
		setCallStatus(ToolCallStatus.CALL_CALLING);
		String id=getID();
		logger.log(Logger.TRACE,"Tool.call("+id+")"); //$NON-NLS-1$ //$NON-NLS-2$
		String caller=getCgiRequest();
		if("".equals(caller)) { //$NON-NLS-1$
			// Proper call status has been set in getCgiRequest
			return;
		}
		
		// XXX should set output_format as resultlocation, get location,
		// and wait til unlocked to get digest, so that no http behaviour pb.
		
		URL url=null;
		URLConnection urlc=null;
		String mime;
		try {
			url=new URL(caller);
			setCallStatus(ToolCallStatus.CALL_CONNECTING);
			// XXX iokdbg: try to fix timeout, with sthg like:
			// System.setProperty ("sun.net.client.defaultReadTimeout","30000");
			// @see http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4143518
			urlc=url.openConnection();
			urlc.setUseCaches(false);
			urlc.setRequestProperty("User-Agent",conf.getUserAgent()); //$NON-NLS-1$
		}
		catch(MalformedURLException mue) {
			logger.log(Logger.ERROR, "Tool.call: Malformed Caller Url"); //$NON-NLS-1$
			logger.log(Logger.DEBUG, "Tool.call: caller="+caller); //$NON-NLS-1$
			setCallStatus(ToolCallStatus.CALL_BAD_URL);
			return;
		}
		catch(IOException ioe) {
			logger.log(Logger.ERROR, "Tool.call: Error connecting"); //$NON-NLS-1$
			logger.log(Logger.DEBUG, "Tool.call: caller="+caller); //$NON-NLS-1$
			setCallStatus(ToolCallStatus.CALL_CONNECTION_ERROR);
			return;
		}
		
		boolean called_ok=false;
		try {
			setCallStatus(ToolCallStatus.CALL_GETCONTENT);
			digest=(String)urlc.getContent();
		}
		catch(IOException ioe) {
			logger.log(Logger.ERROR,"Tool.call: Error getting result()"); //$NON-NLS-1$
			String ioerr=ioe.getMessage();
			logger.log(Logger.DEBUG,"Tool.call: "+ioerr); //$NON-NLS-1$
			if(ioerr.indexOf("HTTP response code: 500")!=-1) { //$NON-NLS-1$
				setCallStatus(ToolCallStatus.CALL_GETCONTENT_SERVER_ERROR);
			}
			else {
				setCallStatus(ToolCallStatus.CALL_GETCONTENT_ERROR);
			}
			return;
		}
		catch(Exception e) {
			logger.log(Logger.DEBUG, "Tool.call: UnsupportedMime"); //$NON-NLS-1$
			// XXX: handle other mime types if any
			setCallStatus(ToolCallStatus.CALL_MIMETYPE_ERROR);
			return;
		}

		setCallStatus(ToolCallStatus.CALL_PARSERESULTS);
		called_ok=this.parseDigest();
		if(!called_ok) {
			logger.log(Logger.DEBUG, "Tool.call: !parseDigest"); //$NON-NLS-1$
			setCallStatus(ToolCallStatus.CALL_PARSERESULTS_ERROR);
			return;
		}
		setCallStatus(ToolCallStatus.CALL_OK);
	}
	
	private String idToIdOp(String id) {
		Logger logger=Logger.getInstance();
		String gid=id;
		int idx=id.indexOf('.');
		if(idx==-1) {
//			logger.log(
//					Logger.WARNING,
//					"Tool.idToIdOp("+id+") has no dot suffix");
		}
		else {
			gid=id.substring(0,idx);
		}
		return gid;
	}
	/**
	 * Set tool ID
	 * @param id The id to set.
	 */
	public void setIdOT(String id) {
		this.id=id;
		super.putAtt("idop",idToIdOp(id)); //$NON-NLS-1$
	}
	
	private class ObservableTool extends Observable implements Serializable {
		private Tool t;
		ObservableTool(Tool t) {
			this.t=t;
		}
		void ping() {
			setChanged();
			notifyObservers(t);
		}
	}
	
	
	public void addObserver(Observer o) {
		obs.addObserver(o);
	}
	public void deleteObserver(Observer o) {
		obs.deleteObserver(o);
	}
}
