package cds.aida.tool;
/* $Id: ToolsList.java,v 1.2 2007/06/06 12:24:19 beugnot Exp $ */
import java.awt.Color;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Observer;
import java.util.Observable;
import java.util.Set;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import cds.aida.gui.Conf;
import cds.aida.gui.L10N;
import cds.aida.gui.Logger;
import cds.aida.wsJlow.ToStringMod;
import cds.jlow.client.descriptor.Attributs;
import cds.jlow.client.descriptor.Registerer;
import cds.jlow.client.util.ToHtmlModule;
import cds.jlow.client.view.StringViewModule;
import cds.jlow.util.Type;

/**
 * Model for the list of the available tools
 * and method to fetch this list
 */
public class ToolsList {
	private DefaultTreeModel tree;
	private List list;
	Registerer register;
	Attributs attsTask,attsPort,attsData;

	private ObservableToolsList obs;

	public ToolsList(){
		super();
		obs=new ObservableToolsList(this);
	}
	
	/**
	 * clear the available tools
	 * @return
	 */
	public void resetList() {
		// tools tree
		DefaultMutableTreeNode root=
			new DefaultMutableTreeNode(
					L10N.getInstance().get("Tools.rootname")); //$NON-NLS-1$
		tree=new DefaultTreeModel(root);
		list=new ArrayList();
		obs.ping();
	}

	/**
	 * fetch the available tools list
	 * from the aida server
	 * @return fetching success
	 */
	public boolean fetchList() {
		Logger logger=Logger.getInstance();
		Conf conf=Conf.getInstance();
		
		// URL to get tools list from:
		URL u=null;
		if("dummy".equals(conf.get(Conf.SERVER))) {
			// either the dummy list embedded, for demos
			u=this.getClass().getResource("DummyToolsList.txt");
			logger.log(Logger.WARNING,"Tools.fetchList: Dummy list");
		}
		else {
			// or a real aida cgi url 
			u=conf.getAidaCgi(Conf.CGI_GETLIST);
			if(u==null) {
				// TODO: user config error info
				logger.log(
						Logger.ERROR,
						"Tools.getList: bad url("+u+")");  //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}
		// System.out.println("IOKDBG: Tools.fetchList: url="+u.toString());

		URLConnection urlc=null;
		try {
			// TODO: user working state info
			urlc=u.openConnection();
			urlc.setUseCaches(false);
			urlc.setRequestProperty("User-Agent", conf.getUserAgent()); //$NON-NLS-1$
		}
		catch(IOException ioe) {
			// TODO: delayed user info for error and ConnectException timeout
			logger.log(
					Logger.ERROR,
					"Tools.getList: io error on url("+u+")"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		BufferedReader br=null;
		try {
			br=new BufferedReader(
					new InputStreamReader(
							urlc.getInputStream()));
		}
		catch(IOException ioe) {
			logger.log(
					Logger.ERROR,
					"Tools.getList: inputstream io error for("+u+")"); //$NON-NLS-1$ //$NON-NLS-2$
			//ioe.printStackTrace();
			return false;
		}

		return readNewToolsList(br);
	}
	
	
	
	/**
	 * Read the tools list, i.e. the queue of tools descriptors,
	 * Make every Tool with its descriptor parsed, adding them to the list.
	 * If a problem occur, set the previous tools list back and return false
	 * @param br Reader where to read the list of descriptors
	 * @return true if new list read and parse correctly
	 */
	protected boolean readNewToolsList(BufferedReader br) {
		Logger logger=Logger.getInstance();
		DefaultTreeModel backuptree=tree;
		resetList();

		try {
			String line=br.readLine();
			String desc=""; //$NON-NLS-1$
			String tid=""; //$NON-NLS-1$
			while(line!=null) {

				// skip comments or blanks
				line=line.trim();
				if(line.equals("")||line.startsWith("#")) { //$NON-NLS-1$ //$NON-NLS-2$
					line=br.readLine();
					continue;
				}
				
				// tool.id encountered -> start of a new descriptor
				if(line.startsWith("tool.id")) { //$NON-NLS-1$
					if(!tid.equals("")) { //$NON-NLS-1$
						// unless its the first tool.id,
						// we make the tool from the aggregated desc
						// and reset desc to aggregate new tool.
						Tool t=new Tool(tid);
						t.parseDesc(desc);
						this.addTool(t);
						desc=""; //$NON-NLS-1$
					}
					String kv[]=line.split("="); //$NON-NLS-1$
					tid=kv[1].trim();
					logger.log(
							Logger.TRACE,
							"Tools.getList: desc("+tid+")"); //$NON-NLS-1$ //$NON-NLS-2$
					line=br.readLine();
					continue;
				}
				
				// aggregate desc line, unless we dont have a tool.id yet 
				if(!tid.equals("")) { //$NON-NLS-1$
					desc+=line+"\n"; //$NON-NLS-1$
				}
				else {
					logger.log(
							Logger.WARNING,
							"Tools.getList: skip line("+line+")"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				// and go on...
				line=br.readLine();
			}
			if(!tid.equals("")) { //$NON-NLS-1$
				Tool t=new Tool(tid);
				t.parseDesc(desc);
				this.addTool(t);
			}
			else {
				logger.log(
						Logger.WARNING,
						"Tools.getList: no tool.id found"); //$NON-NLS-1$
			}
		}
		catch(IOException ioe) {
			logger.log(
					Logger.WARNING,
					"Tools.getList: error, unset new list" //$NON-NLS-1$
					);
			tree=backuptree;
			return false;
		}
		initTFW();
		return true;
	}
	
	/**
	 * 
	 * @param id
	 * @return the Tool of given id, or null if not found.
	 */
	public Tool getTool(String id) {
		Tool tool=null;
		List list=getList();
		Iterator i=list.iterator();
		while(i.hasNext()) {
			Object o=i.next();
			try {
				Tool t=(Tool)o;
				if(t.getID().equals(id)) {
					tool=t;
					break;
				}
			}
			catch(ClassCastException cce) {
				Logger.getInstance().log(
						Logger.ERROR,
						"Tools.getTool("+id+"), list as a !Tool("+ //$NON-NLS-1$ //$NON-NLS-2$
						o.getClass().toString()+")"); //$NON-NLS-1$
			}
		}
		return tool;
	}
	
	private void addTool(Tool t) {
		Logger.getInstance().log(
				Logger.TRACE,
				"Tools.addTool("+t.getName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		list.add(t);
		// TODO: update categ ?

		// XXX (maybe better if done in ToolsView)
		DefaultMutableTreeNode node=(DefaultMutableTreeNode)tree.getRoot();
		node.add(new DefaultMutableTreeNode(t));
		obs.ping();
	}
	
	/**
	 * @return the list of tools as a tree.
	 */
	public DefaultTreeModel getTree() {
		return tree;
	}
	
	/**
	 * @return the list of tools as a List
	 */
	public List getList() {
		return list;		
	}
	
	public void addObserver(Observer o) {
		obs.addObserver(o);
	}
	public void deleteObserver(Observer o) {
		obs.deleteObserver(o);
	}
	
	private class ObservableToolsList extends Observable {
		private ToolsList tools;
		public ObservableToolsList(ToolsList tools) {
			this.tools=tools;
		}
		public void ping() {
			setChanged();
			notifyObservers(tools);
		}
	}
	
	public Registerer getReg(){
		return register;
	}
	
	public void initTFW(){
		L10N lion=L10N.getInstance();
		String name,idTask,idPort,idData;
		int r,g,b,x,y;
		Color color;
		Dimension d;
		register = new Registerer();
		
		// TOOL ATTRIBUTS
		name="IOKDBG-ToolAtts"; //$NON-NLS-1$
		idTask=Tool.class.toString();
		color=new Color(141, 90, 99, 200); //$NON-NLS-1$
		d=lion.getDimension("graph.task.size",new Dimension(84,126)); //$NON-NLS-1$
		attsTask=new Attributs(idTask,name,color);
		attsTask.putAtt(Attributs.SIZE,d);
		attsTask.putAtt("border", BorderFactory.createRaisedBevelBorder());
		register.putAtt(idTask,attsTask);
		
		// PORT ATTRIBUTS
		name="IOKDBG-PortAtts"; //$NON-NLS-1$
		
		idPort=ToolPort.class.toString();
		System.out.println("idPort  "+idPort);
		color=lion.getColor("graph.port.color",new Color(242,184,184)); //$NON-NLS-1$
		d=lion.getDimension("graph.port.size",new Dimension(21,21)); //$NON-NLS-1$
		attsPort=new Attributs(idPort,name,color);
		attsPort.putAtt(Attributs.SIZE,d);
		attsPort.putAtt("border", BorderFactory.createRaisedBevelBorder());
		register.putAtt(idPort,attsPort);
		
		
		// DATA ATTRIBUTS
		name="IOKDBG-DataAtts"; //$NON-NLS-1$
		idData="java.data";
		color=lion.getColor("graph.data.color",new Color(242,184,184)); //$NON-NLS-1$
		attsData=new Attributs(idData,name,color);
		color=lion.getColor("graph.data.empty.color",new Color(64,64,242)); //$NON-NLS-1$
		attsData.putAtt(Attributs.EMPTYDATACOLOR,color);
		color=lion.getColor("graph.data.done.color",new Color(64,242,64)); //$NON-NLS-1$
		attsData.putAtt(Attributs.FILLDATACOLOR,color);
		d=lion.getDimension("graph.data.size",new Dimension(32,32)); //$NON-NLS-1$
		attsData.putAtt(Attributs.SIZE,d);
		register.putAtt(idData,attsData);

		
		Hashtable mods = new Hashtable();
//        mods.put("tostring", new ToStringModule());
        mods.put("tohtml", new ToHtmlModule());
        mods.put("tostring", new ToStringMod());
        
		List list=getList();
		Iterator it=list.iterator();
		// for each tool, put its descriptor and its ports descriptors
		// into the register, and link the task and ports

		while(it.hasNext()) {
			Tool tool=null;
			try {
				tool=(Tool)it.next();
			}
			catch(ClassCastException cce) {
				Logger.getInstance().log(
						Logger.WARNING,
						"WSWF.setReg: not a Tool in tools.getList()" //$NON-NLS-1$
						);
				continue;
			}
			//reg.add(tool, idTask);
			register.put(tool.getID(), tool, attsTask);
			tool.setModules(mods);
			//System.out.println("ID : "+tool.getID());
			//System.out.println("id : "+tool.getId());
			/*
			reg.putDescriptor(
					tool.getId(), // corresponding to ToolDataFlavor as string
					tool // which is an ITaskDescriptor
					);
*/			
			System.out.println("ID inputs : "+tool.getID());
			Vector io;

			Iterator itio;
			io=tool.getListIPort();
			itio=io.iterator();
			for(int count=1;itio.hasNext();count++) {
				ToolPort tp = (ToolPort)itio.next();
				String id=tool.getID()+".input."+count; //$NON-NLS-1$
				
				String typt=tp.getData().getType().getName();
				tp.getData().putModule("view", new StringViewModule());
				
				System.out.println("type : "+typt);

				tp.setModules(mods);
				
				
				
				
				register.put(id,tp,chooseAtt(tp,typt));
				
				tool.addInputPort(id);
				
			}
			System.out.println("outputs : ");
			io=tool.getListOPort();
			itio=io.iterator();
			for(int count=1;itio.hasNext();count++) {
				String id=tool.getID()+".output."+count; //$NON-NLS-1$
				ToolPort desc=(ToolPort)itio.next();

				String typt=desc.getData().getType().getName();
				System.out.println("type : "+typt);
				
				desc.setModules(mods);
				register.put(id, desc, chooseAtt(desc,typt));
				
				tool.addOutputPort(id);
				
			}
			
			Logger.getInstance().log(
					Logger.TRACE,
					"WSWF.setReg("+tool.getLabel()+")"); //$NON-NLS-1$ //$NON-NLS-2$*/
		}
	}
	
	public Attributs chooseAtt(ToolPort tp,String type) {
		Attributs portatts;
		portatts = (Attributs) attsPort.clone();
		
		
		
		String t = type.toLowerCase();
	
		if(t.startsWith("fits")) {
			portatts.setId(attsPort.getId()+"fits");
			portatts.setName(attsPort.getName()+"fits");
			portatts.setBGColor(new Color(255,102,102,200));
			tp.getData().setType(new Type("fits"));
		}
		else if(t.startsWith("numeric")) {
			if(t.startsWith("numeric:int")){
				portatts.setId(attsPort.getId()+"numeric:int");
				portatts.setName(attsPort.getName()+"numeric:int");
				portatts.setBGColor(new Color(255,255,102,200));
			}else if(t.startsWith("numeric:float")){
				portatts.setId(attsPort.getId()+"numeric:float");
				portatts.setName(attsPort.getName()+"numeric:float");
				portatts.setBGColor(new Color(255,204,102,200));
			}else{
				portatts.setId(attsPort.getId()+"numeric");
				portatts.setName(attsPort.getName()+"numeric");
				portatts.setBGColor(new Color(255,255,153,200));
			}
		}
		else if(t.startsWith("data")) {
			portatts.setId(attsPort.getId()+"data");
			portatts.setName(attsPort.getName()+"data");
			portatts.setBGColor(new Color(153,255,255,200));
		}
		else if(t.startsWith("image")) {
			portatts.setId(attsPort.getId()+"image");
			portatts.setName(attsPort.getName()+"image");
			portatts.setBGColor(new Color(153,255,153,200));
		}
		else if(t.startsWith("bool")) {
			portatts.setId(attsPort.getId()+"bool");
			portatts.setName(attsPort.getName()+"bool");
			portatts.setBGColor(new Color(255,153,255,200));
			tp.getData().setType(Type.BOOLEAN);
		}
		return portatts;
	}
}
