package cds.aida.w1;
/* $Id: DefaultToolView.java,v 1.4 2007/06/06 12:24:19 beugnot Exp $ */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.util.Observable;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.Scrollable;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import cds.aida.gui.L10N;
import cds.aida.gui.Logger;
import cds.aida.gui.ToolW1View;
import cds.aida.tool.Tool;
import cds.aida.tool.ToolPort;
import cds.jlow.descriptor.Constant;
import cds.jlow.descriptor.IDescriptor;
import cds.jlow.descriptor.IPortDescriptor;
import cds.jlow.descriptor.PortDescriptor;
import cds.jlow.util.DescriptorFactory;
import cds.jlow.util.Type;

/**
 * GUI for a given tool
 */
public class DefaultToolView extends ToolView implements Scrollable {
	// Tool t; inherited from ToolView
	String callstatus_prefix;
	//
	JPanel header;
	JTextArea comment;
	JLabel callstatus;
	Box ins,outs;
	ToolW1View viewstderr,viewstdout;
	//ToolIOView viewstderr,viewstdout;
	private Border compborder=BorderFactory.createEmptyBorder(10,10,10,10);

	public DefaultToolView(Tool t) {
		this.t=t;
		this.setLayout(new BorderLayout());
		this.add(createToolUI(),BorderLayout.CENTER);
		t.addObserver(this);
		setAutoscrolls(true);
	}
	
	/**
	 * Create the view panel for the tool,
	 * dynamically creating views for each of its in/outputs
	 * @return the view panel
	 */
	protected JPanel createToolUI() {
		JPanel p=new JPanel();
		L10N lion=L10N.getInstance();
		p.setOpaque(false);
		p.setBorder(compborder);
		p.setLayout(new BorderLayout());
		
		header=new JPanel();//header=new Box(BoxLayout.Y_AXIS);
		header.setLayout(new BorderLayout());
		header.setOpaque(false);

		createCallStatus();
		setCallStatus();
		createLabel();
		setLabel();
		createComment();
		setComment();
		
		Box io=new Box(BoxLayout.Y_AXIS);
		io.setOpaque(false);
		// ins
		ins=new Box(BoxLayout.Y_AXIS);
		ins.setOpaque(false);
		String institle=lion.get("tool.inputs.label","Inputs"); //$NON-NLS-1$ //$NON-NLS-2$
		ins.setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder(institle),
						BorderFactory.createEmptyBorder(5,5,5,5)
				)
		);
		io.add(ins);
		// outs
		outs=new Box(BoxLayout.Y_AXIS);
		outs.setOpaque(false);
		String outstitle=lion.get("tool.outputs.label","Outputs"); //$NON-NLS-1$ //$NON-NLS-2$
		outs.setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder(outstitle),
						BorderFactory.createEmptyBorder(5,5,5,5)));
		io.add(outs);
		
		int nb_i=t.countInput();
		int nb_o=t.countOutput();
		int it;
		// each tool input
		it=1;
		while(it<=nb_i) {
			ToolPort ti=t.getInput(it);
			ins.add(DefaultToolView.createIOUI(ti));
			it++;
		}
		// each tool output
		it=1;
		while(it<=nb_o) {
			ToolPort to=t.getOutput(it);
			outs.add(DefaultToolView.createIOUI(to));
			it++;
		}
		viewstdout=DefaultToolView.createIOUI(t.stdout());
		viewstderr=DefaultToolView.createIOUI(t.stderr());
		setStdViews();

		p.add(header,BorderLayout.NORTH);
		p.add(io,BorderLayout.CENTER);
		return p;
	}
	protected void createLabel() {
		// label only on header's border 
	}
	protected void clearLabel() {
		header.setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createEmptyBorder(5,5,5,5),
						BorderFactory.createBevelBorder(
								BevelBorder.LOWERED
						)
				)
		);
	}
	protected void setLabel() {
		header.setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createEmptyBorder(5,5,5,5),
						BorderFactory.createTitledBorder(
								BorderFactory.createBevelBorder(
										BevelBorder.LOWERED
								),
								t.getName(),
								TitledBorder.DEFAULT_JUSTIFICATION,
								TitledBorder.ABOVE_TOP
						)
				)
		);
	}
	protected void createComment() {
		comment=new JTextArea(""); //$NON-NLS-1$
		comment.setDropTarget(null);
		comment.setLineWrap(true);
		comment.setWrapStyleWord(true);
		comment.setOpaque(false);
		comment.setEditable(false);
		comment.setBorder(compborder);
	}
	protected void clearComment() {
		header.remove(comment);
		comment.setText(""); //$NON-NLS-1$
	}
	protected void setComment() {
		clearComment();
		String c=t.getComment();
		comment.setText(c);
		if(c!=null&&!c.equals("")) { //$NON-NLS-1$
			header.add(comment, BorderLayout.CENTER);
		}
	}
	protected void createCallStatus() {
		callstatus_prefix=L10N.getInstance().get(
				"tool.call.name", //$NON-NLS-1$
				"Tool call"); //$NON-NLS-1$
		callstatus=new JLabel(""); //$NON-NLS-1$
		Font font=callstatus.getFont();
		Font itfont=font.deriveFont(Font.ITALIC,13f); // iokdbg hardcoded
		callstatus.setFont(itfont);
	}
	protected void clearCallStatus() {
		header.remove(callstatus);
	}
	/**
	 * update the tool's view call status from the tool's status, 
	 * which might popup an information message.
	 */
	protected void setCallStatus() {
		clearCallStatus();
		ToolCallStatus tcs=t.getCallStatus();
		String msg=tcs.toString();
		Logger.getInstance().log(
				Logger.TRACE,
				"DTV.setCallStatus("+tcs+")"); //$NON-NLS-1$ //$NON-NLS-2$
		callstatus.setText(callstatus_prefix+": "+msg); //$NON-NLS-1$
		if(msg!=null && !msg.equals("")) { //$NON-NLS-1$
			header.add(callstatus, BorderLayout.SOUTH);
		}
		if(
				// which status has to be shown as popup
				tcs==ToolCallStatus.CALL_INPUT_NOT_READY
				||tcs==ToolCallStatus.CALL_GETCONTENT_ERROR
				||tcs==ToolCallStatus.CALL_GETCONTENT_SERVER_ERROR
				||tcs==ToolCallStatus.CALL_MIMETYPE_ERROR
				||tcs==ToolCallStatus.CALL_PARSERESULTS_ERROR
		) {
			String popmsg=tcs.toString();
			String poptitle=callstatus_prefix+" -> "+t.getName(); //$NON-NLS-1$
			int tcstype=tcs.getType();
			int poptype;
			switch(tcstype) {
			case ToolCallStatus.INFO:
				poptype=JOptionPane.INFORMATION_MESSAGE;
			break;
			case ToolCallStatus.WARNING:
				poptype=JOptionPane.WARNING_MESSAGE;
			break;
			case ToolCallStatus.ERROR:
				poptype=JOptionPane.ERROR_MESSAGE;
			break;
			default:
				poptype=JOptionPane.INFORMATION_MESSAGE;
			}
			JOptionPane.showMessageDialog(
					null,
					popmsg,
					poptitle,
					poptype
					);
		}
	}
	/**
	 * Display stdout/err views only if isReady()
	 */
	public void setStdViews() {
		outs.remove(viewstdout);
		outs.remove(viewstderr);
		if(t.stdout().isReady()) {
			outs.add(viewstdout);
		}
		if(t.stderr().isReady()){
			outs.add(viewstderr);
		}
	}
	public void update(Observable o,Object oo) {
		Logger.getInstance().log(
				Logger.TRACE,
				"DTV.update()"); //$NON-NLS-1$
		// upate graphical parts related to Tool data,
		// dont care about ToolIOViews because they observe
		// their ToolIO models, but take special care of stdout/err views
		setComment();
		setLabel();
		setCallStatus();
		setStdViews();
	}
	public static ToolW1View createIOUI(ToolPort tp) {
		ToolW1View tv=null;
		// XXX dont like 'instanceof' but dbldispatching pb
		Constant c = tp.getData();
		if(c==null){
			tv=new ToolPortViewDummy(tp);
		}else{
			String t1 = c.getType().getName();
			String t = t1.toLowerCase();
		
			if(t.startsWith("string")) {
				tv=new ToolPortViewString(tp);
			}
			else if(t.startsWith("numeric")) {
				tv=new ToolPortViewString(tp);
			}
			else if(t.startsWith("data")||t.startsWith("fits")||t.startsWith("image")) {
				tv=new ToolPortViewFiles(tp);
			}
			else if(t.startsWith("bool")) {
				tv=new ToolPortViewString(tp);
			}else{
				System.out.println("type : "+t1);
				tv=new ToolPortViewDummy(tp);
			}
		}
		
		return tv;
	}
	
	public Dimension getPreferredScrollableViewportSize() {
		// XXX chk if this value is the best we can give.
		return getPreferredSize();
	}
	public int getScrollableBlockIncrement(
			Rectangle visibleRect,
			int orientation,
			int direction) {
		// TODO Auto-generated method stub
		return 0;
	}
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	public boolean getScrollableTracksViewportWidth() {
		return true;
	}
	public int getScrollableUnitIncrement(
			Rectangle visibleRect,
			int orientation,
			int direction) {
		// TODO Auto-generated method stub
		return 0;
	}
}
