package cds.aida.w1;
/* $Id: ToolCallStatus.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */

import java.io.Serializable;

import cds.aida.gui.L10N;


/**
 * Gives informations about tool call state,
 * and textual informations about it (see L10N properties tool.call.*)
 * @see Tool.call
 */
public class ToolCallStatus implements Serializable {
	String prop, def;
	
	public static final int STATE_FINISHED=1;
	public static final int STATE_RUNNING=2;
	int state;
	
	public static final int INFO=1;
	public static final int WARNING=2;
	public static final int ERROR=3;
	int type;
	
	// predefined status
	
	public static final ToolCallStatus CALL_NOTCALLED=
		new ToolCallStatus(
				"tool.call.notcalled", //$NON-NLS-1$
				"Not yet called", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_CALLING=
		new ToolCallStatus(
				"tool.call.calling", //$NON-NLS-1$
				"Calling...", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_RUNNING
		);
	/**
	 * Execution ended, results parsed, no error found.
	 */
	public static final ToolCallStatus CALL_OK=
		new ToolCallStatus(
				"tool.call.ok", //$NON-NLS-1$
				"Called OK", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_FINISHED
		);
	/**
	 * Execution ended, results parsed ok, but results contain
	 * execution error info.
	 */
	public static final ToolCallStatus CALL_OK_WITH_ERROR=
		new ToolCallStatus(
				"tool.call.okwitherror", //$NON-NLS-1$
				"Tool called, but error appeared", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_FINISHED
		); 
	public static final ToolCallStatus CALL_INPUT_NOT_READY=
		new ToolCallStatus(
				"tool.call.error.input", //$NON-NLS-1$
				"Input not ready", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_BAD_URL=
		new ToolCallStatus(
				"tool.call.error.badurl", //$NON-NLS-1$
				"Bad Url (check usetool parameter in config)", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_CONNECTING=
		new ToolCallStatus(
				"tool.call.connecting", //$NON-NLS-1$
				"Connecting...", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_RUNNING
		);
	public static final ToolCallStatus CALL_CONNECTION_ERROR=
		new ToolCallStatus(
				"tool.call.error.connecting", //$NON-NLS-1$
				"Error during connection", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_GETCONTENT=
		new ToolCallStatus(
				"tool.call.getcontent", //$NON-NLS-1$
				"Waiting results...", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_RUNNING
		);
	public static final ToolCallStatus CALL_GETCONTENT_ERROR=
		new ToolCallStatus(
				"tool.call.error.getcontent", //$NON-NLS-1$
				"Error when fetching results", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_GETCONTENT_SERVER_ERROR=
		new ToolCallStatus(
				"tool.call.error.server500", //$NON-NLS-1$
				"Server error when fetching results", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_MIMETYPE_ERROR=
		new ToolCallStatus(
				"tool.call.error.mimetype", //$NON-NLS-1$
				"Error fetching content, wrong mime type", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	public static final ToolCallStatus CALL_PARSERESULTS=
		new ToolCallStatus(
				"tool.call.parseresults", //$NON-NLS-1$
				"Parsing results", //$NON-NLS-1$
				ToolCallStatus.INFO,
				ToolCallStatus.STATE_RUNNING
		);
	/**
	 * Error when trying to parse the results content
	 */
	public static final ToolCallStatus CALL_PARSERESULTS_ERROR=
		new ToolCallStatus(
				"tool.call.error.parseresults", //$NON-NLS-1$
				"Error results badly formed", //$NON-NLS-1$
				ToolCallStatus.ERROR,
				ToolCallStatus.STATE_FINISHED
		);
	
	/**
	 * Const. with default default value = property key 
	 */
	public ToolCallStatus(String prop) {
		this(prop,prop);
	}
	/**
	 * Const. with default type: INFO 
	 */
	public ToolCallStatus(String prop, String def) {
		this(prop,def,INFO);
	}
	/**
	 * Const. with default state: STATE_FINISHED
	 */
	public ToolCallStatus(String prop, String def, int type) {
		this(prop,def,type,ToolCallStatus.STATE_FINISHED);
	}
	/**
	 * 
	 * @param prop Property key to recover status string in L10N properties
	 * @param def Default value if L10N can't find 'prop' among properties
	 * @param type the status is an INFO, WARNING or ERROR
	 * @param state indicate if the call is STATE_FINISHED or STATE_RUNNING
	 */
	public ToolCallStatus(String prop, String def, int type, int state) {
		this.prop=prop==null?"":prop; //$NON-NLS-1$
		this.def=def==null?"":def; //$NON-NLS-1$
		setType(type);
		this.state=state;
	}
	public void setType(int type) {
		this.type=type;
	}
	public int getType() {
		return type;
	}
	/**
	 * @return true is the status correspond to one the finished states.
	 */
	public boolean isFinished() {
		return state==STATE_FINISHED;
	}
	/**
	 * @return Status message
	 */
	public String toString() {
		return L10N.getInstance().get(prop,def);
	}
}
